@@ -1,5 +1,5 @@
 /* NBD client library in userspace
- * Copyright (C) 2013-2021 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -66,7 +66,7 @@ enum mode {
   MODE_VSOCK,                /* --vsock */
 };
 
-static void __attribute__((noreturn))
+static void __attribute__ ((noreturn))
 usage (FILE *fp, int exitcode)
 {
   fprintf (fp,
@@ -109,7 +109,7 @@ fuse_help (const char *prog)
 {
   static struct fuse_operations null_operations;
   const char *tmp_argv[] = { prog, "--help", NULL };
-  fuse_main (2, (char **) tmp_argv, &null_operations, NULL);
+  fuse_main (2, (char **)tmp_argv, &null_operations, NULL);
   exit (EXIT_SUCCESS);
 }
 
@@ -229,8 +229,8 @@ main (int argc, char *argv[])
     case 'C':
       if (sscanf (optarg, "%u", &connections) != 1 ||
           connections < 1 || connections > 1024) {
-        fprintf (stderr, "%s: --connections parameter must be an unsigned integer >= 1\n",
-                 argv[0]);
+        fprintf (stderr, "%s: --connections option-argument must be an "
+                 "unsigned integer >= 1\n", argv[0]);
         exit (EXIT_FAILURE);
       }
       break;
@@ -415,7 +415,7 @@ main (int argc, char *argv[])
       handles_append (&nbd, h); /* reserved above, so can't fail */
     }
   }
-  connections = (unsigned) nbd.len;
+  connections = nbd.len;
   if (verbose)
     fprintf (stderr, "nbdfuse: connections=%u\n", connections);
 
@@ -424,7 +424,7 @@ main (int argc, char *argv[])
     fprintf (stderr, "%s\n", nbd_get_error ());
     exit (EXIT_FAILURE);
   }
-  size = (uint64_t) ssize;
+  size = ssize;
 
   /* If the remote NBD server is readonly, then act as if the '-r'
    * flag was given on the nbdfuse command line.
@@ -481,7 +481,7 @@ main (int argc, char *argv[])
   if (pidfile) {
     fp = fopen (pidfile, "w");
     if (fp) {
-      fprintf (fp, "%ld", (long) getpid ());
+      fprintf (fp, "%ld", (long)getpid ());
       fclose (fp);
     }
   }
