@@ -94,6 +94,7 @@ let name_of_arg =
   | Closure { cbname } ->
      [ sprintf "%s_callback" cbname; sprintf "%s_user_data" cbname ]
   | Enum (n, _) -> [n]
+  | Extent64 _ -> assert false (* only used in extent64_closure *)
   | Fd n -> [n]
   | Flags (n, _) -> [n]
   | Int n -> [n]
@@ -132,6 +133,7 @@ let arg_attr_nonnull =
   | Bool _ | Closure _ | Enum _ | Fd _ | Flags _
   | Int _ | Int64 _ | SizeT _
   | UInt _ | UInt32 _ | UInt64 _ | UIntPtr _ -> [ false ]
+  | Extent64 _ -> assert false (* only used in extent64_closure *)
 
 let optarg_attr_nonnull (OClosure _ | OFlags _) = [ false ]
 
@@ -191,6 +193,7 @@ and print_arg_list' ?(handle = false) ?(types = true) ?(closure_style = Direct)
       | Enum (n, _) ->
          if types then pr "int ";
          pr "%s" n
+      | Extent64 _ -> assert false (* only used in extent64_closure *)
       | Flags (n, _) ->
          if types then pr "uint32_t ";
          pr "%s" n
@@ -305,6 +308,11 @@ and print_cbarg_list' ?(types = true) cbargs =
          pr "%s, " n;
          if types then pr "size_t ";
          pr "%s" len
+      | CBArrayAndLen (Extent64 n, len) ->
+         if types then pr "nbd_extent *";
+         pr "%s, " n;
+         if types then pr "size_t ";
+         pr "%s" len
       | CBArrayAndLen _ -> assert false
       | CBBytesIn (n, len) ->
          if types then pr "const void *";
@@ -373,13 +381,18 @@ let print_fndecl_and_define ?wrap name args optargs ret =
   pr "#define LIBNBD_HAVE_NBD_%s 1\n" name_upper;
   pr "\n"
 
-let print_ns_ctxt ns ns_upper ctxt consts =
-  let ctxt_upper = String.uppercase_ascii ctxt in
+let print_ns_ctxt ns ns_upper ctxt =
+  let ctxt_macro = macro_name ctxt in
+  let ctxt_upper = String.uppercase_ascii ctxt_macro in
   pr "#define LIBNBD_CONTEXT_%s_%s \"%s:%s\"\n"
-    ns_upper ctxt_upper ns ctxt;
-  pr "\n";
-  pr "/* \"%s:%s\" context related constants */\n" ns ctxt;
-  List.iter (fun (n, i) -> pr "#define LIBNBD_%-30s %d\n" n i) consts
+    ns_upper ctxt_upper ns ctxt
+
+let print_ns_ctxt_bits ns ctxt consts =
+  if consts <> [] then (
+    pr "\n";
+    pr "/* \"%s:%s\" context related constants */\n" ns ctxt;
+    List.iter (fun (n, i) -> pr "#define LIBNBD_%-30s %d\n" n i) consts
+  )
 
 let print_ns ns ctxts =
   let ns_upper = String.uppercase_ascii ns in
@@ -388,7 +401,10 @@ let print_ns ns ctxts =
   pr "\n";
   pr "/* \"%s\" namespace contexts */\n" ns;
   List.iter (
-    fun (ctxt, consts) -> print_ns_ctxt ns ns_upper ctxt consts
+    fun (ctxt, _) -> print_ns_ctxt ns ns_upper ctxt
+  ) ctxts;
+  List.iter (
+    fun (ctxt, consts) -> print_ns_ctxt_bits ns ctxt consts
   ) ctxts;
   pr "\n"
 
@@ -477,6 +493,13 @@ let generate_include_libnbd_h () =
   pr "extern int nbd_get_errno (void);\n";
   pr "#define LIBNBD_HAVE_NBD_GET_ERRNO 1\n";
   pr "\n";
+  pr "/* This is used in the callback for nbd_block_status_64.\n";
+  pr " */\n";
+  pr "typedef struct {\n";
+  pr "  uint64_t length;  /* Will not exceed INT64_MAX */\n";
+  pr "  uint64_t flags;\n";
+  pr "} nbd_extent;\n";
+  pr "\n";
   print_closure_structs ();
   List.iter (
     fun (name, { args; optargs; ret }) ->
@@ -743,6 +766,7 @@ let generate_lib_api_c () =
       | Bool _ | Closure _ | Enum _ | Flags _ | Fd _ | Int _
       | Int64 _ | SizeT _
       | SockAddrAndLen _ | UInt _ | UInt32 _ | UInt64 _ | UIntPtr _ -> ()
+      | Extent64 _ -> assert false (* only used in extent64_closure *)
     ) args;
     let indent =
       if may_set_error then (
@@ -766,6 +790,7 @@ let generate_lib_api_c () =
            pr " %s=\\\"%%s\\\" %s=%%zu" n count
         | Closure { cbname } -> pr " %s=%%s" cbname
         | Enum (n, _) -> pr " %s=%%d" n
+        | Extent64 _ -> assert false (* only used in extent64_closure *)
         | Flags (n, _) -> pr " %s=0x%%x" n
         | Fd n | Int n -> pr " %s=%%d" n
         | Int64 n -> pr " %s=%%\"PRIi64\"" n
@@ -804,6 +829,7 @@ let generate_lib_api_c () =
               pr "%s_printable ? %s_printable : \"\", %s" n n count
            | Closure _ -> pr "\"<fun>\""
            | Enum (n, _) -> pr "%s" n
+           | Extent64 _ -> assert false (* only used in extent64_closure *)
            | Flags (n, _) -> pr "%s" n
            | Fd n | Int n | Int64 n | SizeT n -> pr "%s" n
            | SockAddrAndLen (_, len) -> pr "(int) %s" len
@@ -839,6 +865,7 @@ let generate_lib_api_c () =
       | Bool _ | Closure _ | Enum _ | Flags _ | Fd _ | Int _
       | Int64 _ | SizeT _
       | SockAddrAndLen _ | UInt _ | UInt32 _ | UInt64 _ | UIntPtr _ -> ()
+      | Extent64 _ -> assert false (* only used in extent64_closure *)
     ) args;
     pr "  }\n"
   (* Print the trace when we leave a call with debugging enabled. *)
