@@ -44,6 +44,7 @@ and ocaml_arg_to_string = function
   | Closure { cbargs } ->
      sprintf "(%s)" (ocaml_closuredecl_to_string cbargs)
   | Enum (_, { enum_prefix }) -> sprintf "%s.t" enum_prefix
+  | Extent64 _ -> "extent"
   | Fd _ -> "Unix.file_descr"
   | Flags (_, { flag_prefix }) -> sprintf "%s.t list" flag_prefix
   | Int _ -> "int"
@@ -102,6 +103,7 @@ let ocaml_name_of_arg = function
   | BytesPersistOut (n, len) -> n
   | Closure { cbname } -> cbname
   | Enum (n, _) -> n
+  | Extent64 _ -> assert false (* only used in extent64_closure *)
   | Fd n -> n
   | Flags (n, _) -> n
   | Int n -> n
@@ -147,6 +149,9 @@ exception Closed of string
 
 type cookie = int64
 
+type extent = int64 * int64
+(** Length and flags of an extent in {!block_status_64} callback. *)
+
 ";
 
   List.iter (
@@ -183,7 +188,7 @@ type cookie = int64
       pr "val namespace_%s : string\n" ns;
       List.iter (
         fun (ctxt, consts) ->
-          pr "val context_%s_%s : string\n" ns ctxt;
+          pr "val context_%s_%s : string\n" ns (macro_name ctxt);
           List.iter (fun (n, _) ->
               pr "val %s : int32\n" (String.lowercase_ascii n)
           ) consts
@@ -268,6 +273,7 @@ let generate_ocaml_nbd_ml () =
 exception Error of string * Unix.error option
 exception Closed of string
 type cookie = int64
+type extent = int64 * int64
 
 (* Give the exceptions names so that they can be raised from the C code. *)
 let () =
@@ -315,7 +321,7 @@ let () =
       pr "let namespace_%s = \"%s:\"\n" ns ns;
       List.iter (
         fun (ctxt, consts) ->
-          pr "let context_%s_%s = \"%s:%s\"\n" ns ctxt ns ctxt;
+          pr "let context_%s_%s = \"%s:%s\"\n" ns (macro_name ctxt) ns ctxt;
           List.iter (fun (n, i) ->
               pr "let %s = %d_l\n" (String.lowercase_ascii n) i
           ) consts
@@ -498,7 +504,7 @@ let print_ocaml_closure_wrapper { cbname; cbargs } =
   let argnames =
     List.map (
       function
-      | CBArrayAndLen (UInt32 n, _) | CBBytesIn (n, _)
+      | CBArrayAndLen ((UInt32 n | Extent64 n), _) | CBBytesIn (n, _)
       | CBInt n | CBInt64 n
       | CBMutable (Int n) | CBString n | CBUInt n | CBUInt64 n ->
          n ^ "v"
@@ -531,6 +537,14 @@ let print_ocaml_closure_wrapper { cbname; cbargs } =
        pr "%s  %s,\n" indent n;
        pr "%s  %s\n" indent count;
        pr "%s);\n" indent
+    | CBArrayAndLen (Extent64 n, count) ->
+       pr "  %sv = " n;
+       let fncol = output_column () in
+       let indent = spaces fncol in
+       pr "nbd_internal_ocaml_alloc_extent64_array (\n";
+       pr "%s  %s,\n" indent n;
+       pr "%s  %s\n" indent count;
+       pr "%s);\n" indent
     | CBBytesIn (n, len) ->
        pr "  %sv = caml_alloc_initialized_string (%s, %s);\n" n len n
     | CBInt n | CBUInt n ->
@@ -554,7 +568,7 @@ let print_ocaml_closure_wrapper { cbname; cbargs } =
 
   List.iter (
     function
-    | CBArrayAndLen (UInt32 _, _)
+    | CBArrayAndLen ((UInt32 _ | Extent64 _), _)
     | CBBytesIn _
     | CBInt _
     | CBInt64 _
@@ -695,6 +709,7 @@ let print_ocaml_binding (name, { args; optargs; ret }) =
        pr "  %s_callback.free = free_user_data;\n" cbname
     | Enum (n, { enum_prefix }) ->
        pr "  int %s = %s_val (%sv);\n" n enum_prefix n
+    | Extent64 _ -> assert false (* only used in extent64_closure *)
     | Fd n ->
        pr "  /* OCaml Unix.file_descr is just an int, at least on Unix. */\n";
        pr "  int %s = Int_val (%sv);\n" n n
@@ -792,6 +807,7 @@ let print_ocaml_binding (name, { args; optargs; ret }) =
     | UInt32 _
     | UInt64 _
     | UIntPtr _ -> ()
+    | Extent64 _ -> assert false (* only used in extent64_closure *)
   ) args;
 
   pr "  CAMLreturn (rv);\n";
