@@ -124,6 +124,33 @@ let cspan str reject =
   in
   loop 0
 
+(* String.starts_with was added in OCaml 4.13 so we include this for
+ * earlier versions of OCaml.
+ *)
+let string_starts_with ~prefix s =
+  let len_s = String.length s and len_pre = String.length prefix in
+  let rec aux i =
+    if i = len_pre then true
+    else if String.unsafe_get s i <> String.unsafe_get prefix i then false
+    else aux (i + 1)
+  in
+  len_s >= len_pre && aux 0
+
+(* Array.find_map was added in OCaml 4.13 *)
+let array_find_map f a =
+  let n = Array.length a in
+  let rec loop i =
+    if i = n then None
+    else
+      match f (Array.unsafe_get a i) with
+      | None -> loop (succ i)
+      | Some _ as r -> r
+  in
+  loop 0
+
+(* Option module was added in OCaml 4.08 *)
+let option_map f o = match o with None -> None | Some v -> Some (f v)
+
 (* Current output line and column. *)
 let lineno = ref 1 and col = ref 0
 
@@ -151,6 +178,11 @@ let pr fs =
 
 let spaces n = String.make n ' '
 
+(* Convert s to macro name by changing '-' to '_' and eliding ':'. *)
+let macro_name s =
+  let underscore = Str.global_replace (Str.regexp_string "-") "_" s in
+  Str.global_replace (Str.regexp ":") "" underscore
+
 (* Save the current output channel and replace it with a temporary buffer while
  * running ‘code’.  Return the buffer.
  *)
@@ -365,7 +397,8 @@ type comment_style =
   | CStyle | CPlusPlusStyle | HashStyle | OCamlStyle | HaskellStyle
   | PODCommentStyle
 
-let generate_header ?(extra_sources = []) comment_style =
+let generate_header ?(extra_sources = []) ?(copyright = "Red Hat")
+    comment_style =
   let inputs = "generator/generator" :: extra_sources in
   let c = match comment_style with
     | CStyle ->         pr "/* "; " *"
@@ -379,7 +412,7 @@ let generate_header ?(extra_sources = []) comment_style =
   pr "%s %s\n" c (String.concat " " inputs);
   pr "%s ANY CHANGES YOU MAKE TO THIS FILE WILL BE LOST.\n" c;
   pr "%s\n" c;
-  pr "%s Copyright Red Hat\n" c;
+  pr "%s Copyright %s\n" c copyright;
   pr "%s\n" c;
   pr "%s This library is free software; you can redistribute it and/or\n" c;
   pr "%s modify it under the terms of the GNU Lesser General Public\n" c;
@@ -413,7 +446,11 @@ let files_equal n1 n2 =
   | 1 -> false
   | i -> failwithf "%s: failed with error code %d" cmd i
 
-let output_to filename k =
+type formatter =
+  | Gofmt
+  | Rustfmt
+
+let output_to ?(formatter = None) filename k =
   lineno := 1; col := 0;
   let filename_new = filename ^ ".new" in
   let c = open_out filename_new in
@@ -421,7 +458,32 @@ let output_to filename k =
   k ();
   close_out c;
   chan := NoOutput;
-
+  (match formatter with
+  | Some Gofmt ->
+     (match Config.gofmt with
+      |  Some gofmt ->
+          (let cmd = sprintf "%s -w %s" gofmt filename_new in
+           match system cmd with
+           | WEXITED 0 -> ()
+           | WEXITED i -> failwithf "gofmt failed with exit code %d" i
+           | WSIGNALED i | WSTOPPED i ->
+              failwithf "gofmt was killed or stopped by signal %d" i
+          )
+      | None -> ()
+     )
+  | Some Rustfmt ->
+     (match Config.rustfmt with
+      | Some rustfmt ->
+         (let cmd = sprintf "%s %s" rustfmt filename_new in
+          match system cmd with
+          | WEXITED 0 -> ()
+          | WEXITED i -> failwithf "rustfmt failed with exit code %d" i
+          | WSIGNALED i | WSTOPPED i ->
+             failwithf "rustfmt was killed or stopped by signal %d" i
+         )
+      | None -> ()
+     )
+  | None -> ());
   (* Is the new file different from the current file? *)
   if Sys.file_exists filename && files_equal filename filename_new then
     unlink filename_new                 (* same, so skip it *)
@@ -500,3 +562,11 @@ let pod2text longdesc =
     Hashtbl.add cache key lines;
     save_cache ();
     lines
+
+let camel_case name =
+  let xs = nsplit "_" name in
+  List.fold_left (
+    fun a x ->
+      a ^ String.uppercase_ascii (Str.first_chars x 1) ^
+          String.lowercase_ascii (Str.string_after x 1)
+  ) "" xs
