@@ -23,6 +23,11 @@ set -x
 
 requires nbdkit --version
 requires nbdkit file --version
+case $(nbdkit file --version 2>&1) in
+    *1.34.[012]* )
+        echo "$0: skipping known double-free bug in nbdkit file dir="
+        exit 77
+esac
 
 # This test requires nbdkit >= 1.22.
 minor=$( nbdkit --dump-config | grep ^version_minor | cut -d= -f2 )
