@@ -36,13 +36,13 @@
 
 #include "nbdinfo.h"
 
-DEFINE_VECTOR_TYPE (uint32_vector, uint32_t);
+DEFINE_VECTOR_TYPE (extent_vector, nbd_extent);
 
-static void print_extents (uint32_vector *entries);
-static void print_totals (uint32_vector *entries, int64_t size);
+static void print_extents (extent_vector *entries);
+static void print_totals (extent_vector *entries, int64_t size);
 static int extent_callback (void *user_data, const char *metacontext,
                             uint64_t offset,
-                            uint32_t *entries, size_t nr_entries,
+                            nbd_extent *entries, size_t nr_entries,
                             int *error);
 
 void
@@ -50,10 +50,17 @@ do_map (void)
 {
   size_t i;
   int64_t size;
-  uint32_vector entries = empty_vector;
+  extent_vector entries = empty_vector;
   uint64_t offset, align, max_len;
   size_t prev_entries_size;
 
+  /* Map mode requires switching over to transmission phase. */
+  if (nbd_aio_is_negotiating (nbd) &&
+      nbd_opt_go (nbd) == -1) {
+    fprintf (stderr, "%s: %s\n", progname, nbd_get_error ());
+    exit (EXIT_FAILURE);
+  }
+
   /* Did we get the requested map? */
   if (!nbd_can_meta_context (nbd, map)) {
     fprintf (stderr,
@@ -69,14 +76,16 @@ do_map (void)
     fprintf (stderr, "%s: %s\n", progname, nbd_get_error ());
     exit (EXIT_FAILURE);
   }
+  if (nbd_get_extended_headers_negotiated (nbd) == 1)
+    max_len = size;
 
   for (offset = 0; offset < size;) {
     prev_entries_size = entries.len;
-    if (nbd_block_status (nbd, MIN (size - offset, max_len), offset,
-                          (nbd_extent_callback) {
-                            .callback = extent_callback,
-                            .user_data = &entries },
-                          0) == -1) {
+    if (nbd_block_status_64 (nbd, MIN (size - offset, max_len), offset,
+                             (nbd_extent64_callback) {
+                               .callback = extent_callback,
+                               .user_data = &entries },
+                             0) == -1) {
       fprintf (stderr, "%s: %s\n", progname, nbd_get_error ());
       exit (EXIT_FAILURE);
     }
@@ -86,9 +95,8 @@ do_map (void)
                progname);
       exit (EXIT_FAILURE);
     }
-    assert ((entries.len & 1) == 0);
-    for (i = prev_entries_size; i < entries.len; i += 2)
-      offset += entries.ptr[i];
+    for (i = prev_entries_size; i < entries.len; i++)
+      offset += entries.ptr[i].length;
   }
 
   if (!totals)
@@ -99,18 +107,18 @@ do_map (void)
 }
 
 /* Callback handling --map. */
-static void print_one_extent (uint64_t offset, uint64_t len, uint32_t type);
-static void extent_description (const char *metacontext, uint32_t type,
+static void print_one_extent (uint64_t offset, uint64_t len, uint64_t type);
+static void extent_description (const char *metacontext, uint64_t type,
                                 char **descr, bool *free_descr,
                                 const char **fg, const char **bg);
 
 static int
 extent_callback (void *user_data, const char *metacontext,
                  uint64_t offset,
-                 uint32_t *entries, size_t nr_entries,
+                 nbd_extent *entries, size_t nr_entries,
                  int *error)
 {
-  uint32_vector *list = user_data;
+  extent_vector *list = user_data;
   size_t i;
 
   if (strcmp (metacontext, map) != 0)
@@ -120,7 +128,7 @@ extent_callback (void *user_data, const char *metacontext,
    * print_extents below.
    */
   for (i = 0; i < nr_entries; ++i) {
-    if (uint32_vector_append (list, entries[i]) == -1) {
+    if (extent_vector_append (list, entries[i]) == -1) {
       perror ("realloc");
       exit (EXIT_FAILURE);
     }
@@ -129,26 +137,26 @@ extent_callback (void *user_data, const char *metacontext,
 }
 
 static void
-print_extents (uint32_vector *entries)
+print_extents (extent_vector *entries)
 {
   size_t i, j;
   uint64_t offset = 0;          /* end of last extent printed + 1 */
-  size_t last = 0;              /* last entry printed + 2 */
+  size_t last = 0;              /* last entry printed + 1 */
 
   if (json_output) fprintf (fp, "[\n");
 
-  for (i = 0; i < entries->len; i += 2) {
-    uint32_t type = entries->ptr[last+1];
+  for (i = 0; i < entries->len; i++) {
+    uint64_t type = entries->ptr[last].flags;
 
     /* If we're coalescing and the current type is different from the
      * previous one then we should print everything up to this entry.
      */
-    if (last != i && entries->ptr[i+1] != type) {
+    if (last != i && entries->ptr[i].flags != type) {
       uint64_t len;
 
       /* Calculate the length of the coalesced extent. */
-      for (j = last, len = 0; j < i; j += 2)
-        len += entries->ptr[j];
+      for (j = last, len = 0; j < i; j++)
+        len += entries->ptr[j].length;
       print_one_extent (offset, len, type);
       offset += len;
       last = i;
@@ -157,11 +165,11 @@ print_extents (uint32_vector *entries)
 
   /* Print the last extent if there is one. */
   if (last != i) {
-    uint32_t type = entries->ptr[last+1];
+    uint64_t type = entries->ptr[last].flags;
     uint64_t len;
 
-    for (j = last, len = 0; j < i; j += 2)
-      len += entries->ptr[j];
+    for (j = last, len = 0; j < i; j++)
+      len += entries->ptr[j].length;
     print_one_extent (offset, len, type);
   }
 
@@ -169,7 +177,7 @@ print_extents (uint32_vector *entries)
 }
 
 static void
-print_one_extent (uint64_t offset, uint64_t len, uint32_t type)
+print_one_extent (uint64_t offset, uint64_t len, uint64_t type)
 {
   static bool comma = false;
   char *descr;
@@ -185,7 +193,7 @@ print_one_extent (uint64_t offset, uint64_t len, uint32_t type)
       ansi_colour (bg, fp);
     fprintf (fp, "%10" PRIu64 "  "
              "%10" PRIu64 "  "
-             "%3" PRIu32,
+             "%3" PRIu64,
              offset, len, type);
     if (descr)
       fprintf (fp, "  %s", descr);
@@ -199,7 +207,7 @@ print_one_extent (uint64_t offset, uint64_t len, uint32_t type)
 
     fprintf (fp, "{ \"offset\": %" PRIu64 ", "
              "\"length\": %" PRIu64 ", "
-             "\"type\": %" PRIu32,
+             "\"type\": %" PRIu64,
              offset, len, type);
     if (descr) {
       fprintf (fp, ", \"description\": ");
@@ -215,9 +223,9 @@ print_one_extent (uint64_t offset, uint64_t len, uint32_t type)
 
 /* --map --totals suboption */
 static void
-print_totals (uint32_vector *entries, int64_t size)
+print_totals (extent_vector *entries, int64_t size)
 {
-  uint32_t type;
+  uint64_t type;
   bool comma = false;
 
   /* This is necessary to avoid a divide by zero below, but if the
@@ -237,16 +245,16 @@ print_totals (uint32_vector *entries, int64_t size)
    */
   type = 0;
   for (;;) {
-    uint64_t next_type = (uint64_t)UINT32_MAX + 1;
+    uint64_t next_type = 0;
     uint64_t c = 0;
     size_t i;
 
-    for (i = 0; i < entries->len; i += 2) {
-      uint32_t t = entries->ptr[i+1];
+    for (i = 0; i < entries->len; i++) {
+      uint64_t t = entries->ptr[i].flags;
 
       if (t == type)
-        c += entries->ptr[i];
-      else if (type < t && t < next_type)
+        c += entries->ptr[i].length;
+      else if (type < t && (next_type == 0 || t < next_type))
         next_type = t;
     }
 
@@ -263,7 +271,7 @@ print_totals (uint32_vector *entries, int64_t size)
           ansi_colour (fg, fp);
         if (bg)
           ansi_colour (bg, fp);
-        fprintf (fp, "%10" PRIu64 " %5.1f%% %3" PRIu32,
+        fprintf (fp, "%10" PRIu64 " %5.1f%% %3" PRIu64,
                  c, percent, type);
         if (descr)
           fprintf (fp, " %s", descr);
@@ -278,7 +286,7 @@ print_totals (uint32_vector *entries, int64_t size)
         fprintf (fp,
                  "{ \"size\": %" PRIu64 ", "
                  "\"percent\": %g, "
-                 "\"type\": %" PRIu32,
+                 "\"type\": %" PRIu64,
                  c, percent, type);
         if (descr) {
           fprintf (fp, ", \"description\": ");
@@ -292,7 +300,7 @@ print_totals (uint32_vector *entries, int64_t size)
         free (descr);
     }
 
-    if (next_type == (uint64_t)UINT32_MAX + 1)
+    if (next_type == 0)
       break;
     type = next_type;
   }
@@ -301,7 +309,7 @@ print_totals (uint32_vector *entries, int64_t size)
 }
 
 static void
-extent_description (const char *metacontext, uint32_t type,
+extent_description (const char *metacontext, uint64_t type,
                     char **descr, bool *free_descr,
                     const char **fg, const char **bg)
 {
@@ -348,7 +356,7 @@ extent_description (const char *metacontext, uint32_t type,
       *fg = ANSI_FG_BRIGHT_WHITE; *bg = ANSI_BG_BLACK;
       return;
     default:
-      if (asprintf (descr, "backing depth %u", type) == -1) {
+      if (asprintf (descr, "backing depth %" PRIu64, type) == -1) {
         perror ("asprintf");
         exit (EXIT_FAILURE);
       }
