@@ -73,6 +73,8 @@ struct meta_context {
 };
 DEFINE_VECTOR_TYPE (meta_vector, struct meta_context);
 
+DEFINE_VECTOR_TYPE(uint32_vector, uint32_t);
+
 struct export {
   char *name;
   char *description;
@@ -80,12 +82,14 @@ struct export {
 
 struct command_cb {
   union {
-    nbd_extent_callback extent;
+    nbd_extent_callback extent32;
+    nbd_extent64_callback extent64;
     nbd_chunk_callback chunk;
     nbd_list_callback list;
     nbd_context_callback context;
   } fn;
   nbd_completion_callback completion;
+  bool wide;
 };
 
 struct nbd_handle {
@@ -110,6 +114,10 @@ struct nbd_handle {
   char *tls_username;           /* Username, NULL = use current username */
   char *tls_psk_file;           /* PSK filename, NULL = no PSK */
 
+  /* Extended headers. */
+  bool request_eh;              /* Whether to request extended headers */
+  bool extended_headers;        /* If we negotiated NBD_OPT_EXTENDED_HEADERS */
+
   /* Desired metadata contexts. */
   bool request_sr;
   bool request_meta;
@@ -212,6 +220,7 @@ struct nbd_handle {
    * and commands.
    */
   union {
+    uint64_t align_; /* Start sbuf on an 8-byte alignment where useful */
     struct nbd_old_handshake old_handshake;
     struct nbd_new_handshake new_handshake;
     struct nbd_new_option option;
@@ -221,34 +230,48 @@ struct nbd_handle {
         struct {
           struct nbd_fixed_new_option_reply_server server;
           char str[NBD_MAX_STRING * 2 + 1]; /* name, description, NUL */
-        } __attribute__ ((packed)) server;
+        } NBD_ATTRIBUTE_PACKED server;
         struct nbd_fixed_new_option_reply_info_export export;
         struct nbd_fixed_new_option_reply_info_block_size block_size;
         struct {
           struct nbd_fixed_new_option_reply_info_name_or_desc info;
           char str[NBD_MAX_STRING];
-        } __attribute__ ((packed)) name_desc;
+        } NBD_ATTRIBUTE_PACKED name_desc;
         struct {
           struct nbd_fixed_new_option_reply_meta_context context;
           char str[NBD_MAX_STRING];
-        }  __attribute__ ((packed)) context;
+        } NBD_ATTRIBUTE_PACKED context;
         char err_msg[NBD_MAX_STRING];
       } payload;
-    }  __attribute__ ((packed)) or;
+    } or;
     struct nbd_export_name_option_reply export_name_reply;
-    struct nbd_simple_reply simple_reply;
     struct {
-      struct nbd_structured_reply structured_reply;
-      union {
-        struct nbd_structured_reply_offset_data offset_data;
-        struct nbd_structured_reply_offset_hole offset_hole;
+      union reply_header {
+        struct nbd_simple_reply simple;
+        struct nbd_structured_reply structured;
+        struct nbd_extended_reply extended;
+        /* The wire formats share magic and cookie at the same offsets;
+         * provide aliases for one less level of typing.
+         */
         struct {
-          struct nbd_structured_reply_error error;
+          uint32_t magic;
+          uint32_t pad_;
+          uint64_t cookie;
+        };
+      } hdr;
+      union chunk_payload {
+        uint64_t align_; /* Start reply.payload on an 8-byte alignment */
+        struct nbd_chunk_offset_data offset_data;
+        struct nbd_chunk_offset_hole offset_hole;
+        struct nbd_chunk_block_status_32 bs_hdr_32;
+        struct nbd_chunk_block_status_64 bs_hdr_64;
+        struct {
+          struct nbd_chunk_error error;
           char msg[NBD_MAX_STRING]; /* Common to all error types */
           uint64_t offset; /* Only used for NBD_REPLY_TYPE_ERROR_OFFSET */
-        } __attribute__ ((packed)) error;
+        } NBD_ATTRIBUTE_PACKED error;
       } payload;
-    }  __attribute__ ((packed)) sr;
+    } reply;
     uint16_t gflags;
     uint32_t cflags;
     uint32_t len;
@@ -260,7 +283,10 @@ struct nbd_handle {
   /* Issuing a command must use a buffer separate from sbuf, for the
    * case when we interrupt a request to service a reply.
    */
-  struct nbd_request request;
+  union {
+    struct nbd_request compact;
+    struct nbd_request_ext extended;
+  } req;
   bool in_write_payload;
   bool in_write_shutdown;
 
@@ -292,8 +318,21 @@ struct nbd_handle {
   string_vector querylist;
   size_t querynum;
 
+  /* Chunk payload length remaining to be parsed */
+  size_t payload_left;
+
   /* When receiving block status, this is used. */
-  uint32_t *bs_entries;
+  size_t bs_count; /* count of descriptors (not bytes or cooked entries!) */
+  union {
+    char *storage;
+    struct nbd_block_descriptor_32 *narrow;
+    struct nbd_block_descriptor_64 *wide;
+  } bs_raw;
+  union {
+    char *storage;
+    uint32_t *narrow;  /* Note that this array has 2*bs_count entries... */
+    nbd_extent *wide;  /* ...while this is just bs_count entries. */
+  } bs_cooked;
 
   /* Commands which are waiting to be issued [meaning the request
    * packet is sent to the server].  This is used as a simple linked
@@ -361,8 +400,9 @@ struct command {
   uint16_t type;
   uint64_t cookie;
   uint64_t offset;
-  uint32_t count;
-  void *data; /* Buffer for read/write */
+  uint64_t count;
+  void *data; /* Buffer for read/write, uint32_vector* for status payload */
+  uint32_vector *ids; /* For block status with payload */
   struct command_cb cb;
   bool initialized; /* For read, true if getting a hole may skip memset */
   uint32_t data_seen; /* For read, cumulative size of data chunks seen */
