@@ -1,5 +1,5 @@
 /* NBD client library in userspace
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -66,6 +66,7 @@ read_cb (void *opaque,
          uint64_t offset, unsigned status, int *error)
 {
   assert (!read_cb_freed);
+  assert (!completion_cb_called);
   read_cb_called++;
   return 0;
 }
@@ -73,6 +74,7 @@ read_cb (void *opaque,
 static void
 read_cb_free (void *opaque)
 {
+  assert (!completion_cb_freed);
   read_cb_freed++;
 }
 
@@ -81,6 +83,7 @@ block_status_cb (void *opaque, const char *meta, uint64_t offset,
                  uint32_t *entries, size_t nr_entries, int *error)
 {
   assert (!block_status_cb_freed);
+  assert (!completion_cb_called);
   block_status_cb_called++;
   return 0;
 }
@@ -88,6 +91,7 @@ block_status_cb (void *opaque, const char *meta, uint64_t offset,
 static void
 block_status_cb_free (void *opaque)
 {
+  assert (!completion_cb_freed);
   block_status_cb_freed++;
 }
 
@@ -117,6 +121,14 @@ main (int argc, char *argv[])
   struct nbd_handle *nbd;
   int64_t cookie;
   char buf[512];
+  nbd_debug_callback debug_callback = { .callback = debug_fn,
+                                        .free = debug_fn_free };
+  nbd_chunk_callback chunk_callback = { .callback = read_cb,
+                                        .free = read_cb_free };
+  nbd_extent_callback extent_callback = { .callback = block_status_cb,
+                                          .free = block_status_cb_free };
+  nbd_completion_callback completion_callback = { .callback = completion_cb,
+                                                  .free = completion_cb_free };
 
   /* Check debug functions are freed when a new debug function is
    * registered, and when the handle is closed.
@@ -124,13 +136,10 @@ main (int argc, char *argv[])
   nbd = nbd_create ();
   if (nbd == NULL) NBD_ERROR;
 
-  nbd_set_debug_callback (nbd,
-                          (nbd_debug_callback) { .callback = debug_fn,
-                                                 .free = debug_fn_free });
+  nbd_set_debug_callback (nbd, debug_callback);
   assert (debug_fn_freed == 0);
 
-  nbd_set_debug_callback (nbd, (nbd_debug_callback) { .callback = debug_fn,
-                                                      .free = debug_fn_free });
+  nbd_set_debug_callback (nbd, debug_callback);
   assert (debug_fn_freed == 1);
 
   debug_fn_freed = 0;
@@ -142,13 +151,13 @@ main (int argc, char *argv[])
   if (nbd == NULL) NBD_ERROR;
   if (nbd_connect_command (nbd, nbdkit) == -1) NBD_ERROR;
 
-  cookie = nbd_aio_pread_structured (nbd, buf, sizeof buf, 0,
-                                     (nbd_chunk_callback) { .callback = read_cb,
-                                                            .free = read_cb_free },
-                                     (nbd_completion_callback) { .callback = completion_cb,
-                                                                 .free = completion_cb_free },
-                                     0);
+  cookie = nbd_aio_pread_structured (nbd, buf, sizeof buf, 0, chunk_callback,
+                                     completion_callback, 0);
   if (cookie == -1) NBD_ERROR;
+  /* read_cb_called is indeterminate at this point, as state machine
+   * progress may vary based on task schduling and network speed factors.
+   */
+  assert (completion_cb_called == 0);
   assert (read_cb_freed == 0);
   assert (completion_cb_freed == 0);
   while (!nbd_aio_command_completed (nbd, cookie)) {
@@ -172,16 +181,14 @@ main (int argc, char *argv[])
   if (nbd == NULL) NBD_ERROR;
   if (nbd_connect_command (nbd, nbdkit_delay) == -1) NBD_ERROR;
 
-  cookie = nbd_aio_pread_structured (nbd, buf, sizeof buf, 0,
-                                     (nbd_chunk_callback) { .callback = read_cb,
-                                                            .free = read_cb_free },
-                                     (nbd_completion_callback) { .callback = completion_cb,
-                                                                 .free = completion_cb_free },
-                                     0);
+  cookie = nbd_aio_pread_structured (nbd, buf, sizeof buf, 0, chunk_callback,
+                                     completion_callback, 0);
   if (cookie == -1) NBD_ERROR;
   nbd_kill_subprocess (nbd, 0);
   nbd_close (nbd);
 
+  /* read_cb_called is indeterminate based on timing of kill. */
+  assert (completion_cb_called == 1);
   assert (read_cb_freed == 1);
   assert (completion_cb_freed == 1);
 
@@ -195,16 +202,14 @@ main (int argc, char *argv[])
   /* Intentionally omit a call to:
    *  nbd_add_meta_context (nbd, LIBNBD_CONTEXT_BASE_ALLOCATION);
    */
-  cookie = nbd_aio_block_status (nbd, sizeof buf, 0,
-                                 (nbd_extent_callback) { .callback = block_status_cb,
-                                                         .free = block_status_cb_free },
-                                 (nbd_completion_callback) { .callback = completion_cb,
-                                                             .free = completion_cb_free },
-                                 0);
+  cookie = nbd_aio_block_status (nbd, sizeof buf, 0, extent_callback,
+                                 completion_callback, 0);
   if (cookie != -1) {
     fprintf (stderr, "%s: Expecting block_status failure\n", argv[0]);
     exit (EXIT_FAILURE);
   }
+  assert (block_status_cb_called == 0);
+  assert (completion_cb_called == 0);
   assert (block_status_cb_freed == 1);
   assert (completion_cb_freed == 1);
 
@@ -213,16 +218,14 @@ main (int argc, char *argv[])
 
   if (nbd_connect_command (nbd, nbdkit) == -1) NBD_ERROR;
 
-  cookie = nbd_aio_block_status (nbd, sizeof buf, 0,
-                                 (nbd_extent_callback) { .callback = block_status_cb,
-                                                         .free = block_status_cb_free },
-                                 (nbd_completion_callback) { .callback = completion_cb,
-                                                             .free = completion_cb_free },
-                                 0);
+  cookie = nbd_aio_block_status (nbd, sizeof buf, 0, extent_callback,
+                                 completion_callback, 0);
   if (cookie != -1) {
     fprintf (stderr, "%s: Expecting block_status failure\n", argv[0]);
     exit (EXIT_FAILURE);
   }
+  assert (block_status_cb_called == 0);
+  assert (completion_cb_called == 0);
   assert (block_status_cb_freed == 1);
   assert (completion_cb_freed == 1);
 
