@@ -1,5 +1,5 @@
 /* NBD client library in userspace
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,7 +31,7 @@
 
 #define SIZE 65536
 #define XSTR(s) #s
-#define STR(s) XSTR(s)
+#define STR(s) XSTR (s)
 
 static char wbuf[512] = { 1, 2, 3, 4 }, rbuf[512];
 static const char *progname;
@@ -82,8 +82,10 @@ main (int argc, char *argv[])
   struct nbd_handle *nbd;
   int64_t r;
   char *args[] = { "nbdkit", "-s", "-o", "--exit-with-parent", "-v",
-                   "memory", "size=" STR(SIZE), NULL };
+                   "memory", "size=" STR (SIZE), NULL };
   int calls = 0;
+  nbd_chunk_callback chunk_callback = { .callback = pread_cb,
+                                        .user_data = &calls };
   const char *s;
 
   progname = argv[0];
@@ -174,8 +176,7 @@ main (int argc, char *argv[])
   /* Test again for callback operation. */
   memset (rbuf, 0, sizeof rbuf);
   if (nbd_pread_structured (nbd, rbuf, sizeof rbuf, 2 * sizeof rbuf,
-                            (nbd_chunk_callback) { .callback = pread_cb, .user_data = &calls },
-                            0) == -1) {
+                            chunk_callback, 0) == -1) {
     fprintf (stderr, "%s\n", nbd_get_error ());
     exit (EXIT_FAILURE);
   }
@@ -191,8 +192,7 @@ main (int argc, char *argv[])
 
   /* Also test that callback errors are reflected correctly. */
   if (nbd_pread_structured (nbd, rbuf, sizeof rbuf, 2 * sizeof rbuf,
-                            (nbd_chunk_callback) { .callback = pread_cb, .user_data = &calls },
-                            0) != -1) {
+                            chunk_callback, 0) != -1) {
     fprintf (stderr, "%s: expected failure from callback\n", argv[0]);
     exit (EXIT_FAILURE);
   }
